<?php
/**
 * Functions required to start an MIMP session.
 *
 * $Horde: mimp/lib/Session.php,v 1.23 2005/07/03 02:44:59 selsky Exp $
 *
 * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
 * Copyright 1999-2005 Jon Parise <jon@horde.org>
 * Copyright 2004-2005 Michael Slusarz <slusarz@bigworm.curecanti.org>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@horde.org>
 * @author  Michael Slusarz <slusarz@bigworm.curecanti.org>
 * @package MIMP
 */
class MIMP_Session {

    /**
     * Take information posted from a login attempt and try setting up
     * an initial MIMP session. Handle Horde authentication, if
     * required, and only do enough work to see if the user can log
     * in. This function should only be called once, when the user
     * first logs in.
     *
     * @param string $imapuser  The username of the user.
     * @param string $password  The password of the user.
     * @param string $server    The server to use (see config/servers.php).
     * @param array $args       The necessary server information.
     *
     * @return mixed  True on success; string with the failure reason on
     *                failure.
     */
    function createSession($imapuser, $password, $server, $args = array())
    {
        global $conf;

        if (empty($imapuser) || empty($password)) {
            return 'failed';
        }

        /* Make sure all necessary parameters are set. */
        $default_args = array(
            'realm'      => '',
            'port'       => '',
            'protocol'   => '',
            'maildomain' => '',
            'namespace'  => '',
            'folders'    => ''
        );

        /* Merge with the passed-in parameters. */
        $args = array_merge($default_args, $args);

        /* Create the $imp session variable. */
        $_SESSION['mimp'] = array();
        $_SESSION['mimp']['pass'] = Secret::write(Secret::getKey('mimp'), $password);

        /* Run the username through virtualhost expansion functions if
         * necessary. */
        $_SESSION['mimp']['user'] = $imapuser;
        if (!empty($conf['hooks']['vinfo'])) {
            require_once HORDE_BASE . '/config/hooks.php';
            if (function_exists('_mimp_hook_vinfo')) {
                $_SESSION['mimp']['user'] = call_user_func('_mimp_hook_vinfo');
            }
        }

        /* We might need to override some of the defaults with
         * environment-wide settings. Do NOT use the global $servers
         * variable as it may not exist. */
        require MIMP_BASE . '/config/servers.php';

        /* Determine the unique user name. */
        if (Auth::isAuthenticated()) {
            $_SESSION['mimp']['uniquser'] = Auth::getAuth();
        } else {
            $_SESSION['mimp']['uniquser'] = $_SESSION['mimp']['user'];

            if (($conf['server']['server_list'] != 'none') &&
                !empty($servers[$server]['realm'])) {
                $_SESSION['mimp']['uniquser'] .= '@' . $servers[$server]['realm'];
            } elseif (!empty($args['realm'])) {
                $_SESSION['mimp']['uniquser'] .= '@' . $args['realm'];
            }
        }

       if (($conf['server']['server_list'] != 'none') &&
            !empty($servers[$server]) &&
            is_array($servers[$server])) {
            $fields = array('server', 'protocol', 'port', 'folders',
                            'namespace','maildomain', 'dotfiles');
            $fields_array = array('hierarchies');
            $ptr = &$servers[$server];


            foreach ($fields as $val) {
                $_SESSION['mimp'][$val] = isset($ptr[$val]) ? $ptr[$val] : null;
            }
            foreach ($fields_array as $val) {
                $_SESSION['mimp'][$val] = isset($ptr[$val]) ? $ptr[$val] : array();
            }
        } else {
            $server_key = null;
            foreach ($servers as $key => $val) {
                if (is_null($server_key) && (substr($key, 0, 1) != '_')) {
                    $server_key = $key;
                }
                if (MIMP::isPreferredServer($val, $key)) {
                    $server_key = $key;
                    break;
                }
            }
            $ptr = &$servers[$server_key];

            if ($conf['server']['change_server']) {
                $_SESSION['mimp']['server'] = $server;
            } else {
                $_SESSION['mimp']['server'] = $ptr['server'];
            }

            if ($conf['server']['change_port']) {
                $_SESSION['mimp']['port'] = $args['port'];
            } else {
                $_SESSION['mimp']['port'] = $ptr['port'];
            }

            if ($conf['server']['change_protocol']) {
                $_SESSION['mimp']['protocol'] = $args['protocol'];
            } else {
                $_SESSION['mimp']['protocol'] = $ptr['protocol'];
            }

            if ($conf['server']['change_folders']) {
                $_SESSION['mimp']['folders'] = $args['folders'];
            } else {
                $_SESSION['mimp']['folders'] = $ptr['folders'];
            }

            $_SESSION['mimp']['maildomain'] = $args['maildomain'];
            $_SESSION['mimp']['namespace'] = $args['namespace'];
            $_SESSION['mimp']['dotfiles'] = !empty($ptr['dotfiles']);
            $_SESSION['mimp']['hierarchies'] = !empty($ptr['hierarchies']) ? $ptr['hierarchies'] : array();
        }

        /* Determine the base protocol. */
        if (($pos = strpos($_SESSION['mimp']['protocol'], '/'))) {
            $_SESSION['mimp']['base_protocol'] = substr($_SESSION['mimp']['protocol'], 0, $pos);
        } else {
            $_SESSION['mimp']['base_protocol'] = $_SESSION['mimp']['protocol'];
        }

        /* Set the initial mailbox to blank. */
        $_SESSION['mimp']['mailbox'] = '';

        /* Try to authenticate with the given information. */
        $auth_mimp = &Auth::singleton(array('mimp', 'mimp'));
        $auth_mimp->authenticateOptions(array('flags' => OP_HALFOPEN));
        if ($auth_mimp->authenticate(null, null, true) !== true) {
            return false;
        }

        /* Set the session variables. These are cached. */
        $_SESSION['mimp']['mailbox'] = $GLOBALS['prefs']->getValue('mailbox');

        /* IMAP specific variables. */
        if ($_SESSION['mimp']['base_protocol'] != 'pop3') {
            /* Get the delimiter value (per RFC 3501 [6.3.8]). */
            $box = @imap_getmailboxes($_SESSION['mimp']['stream'], MIMP::serverString(), '');
            if (empty($box)) {
                $box = @imap_getmailboxes($_SESSION['mimp']['stream'], MIMP::serverString(), 'INBOX');
            }
            $_SESSION['mimp']['delimiter'] = !empty($box[0]->delimiter) ? $box[0]->delimiter : '/';

            /* Initialize the 'showunsub' value. */
            $_SESSION['mimp']['showunsub'] = false;
        }

        return true;
    }

}
